-- =============================================================================
-- Licensed Materials - Property of IBM
-- 
-- 5724W86
-- 5724X63
-- 
-- (C) Copyright IBM Corp. 2011 All Rights Reserved
-- 
-- US Government Users Restricted Rights
-- - Use, duplication or disclosure restricted by
--   GSA ADP Schedule Contract with IBM Corp.
-- =============================================================================
-- This package must be defined by every pack. It is used by the action
-- framework to refresh the data in the BI tables when needed
-- This code could just as easily be implemented in Java
-- =============================================================================

CREATE OR REPLACE PACKAGE BODY bi_data AS

  -- empty all the DM tables managed in this pack
  PROCEDURE clear
  IS
  BEGIN
    DELETE FROM dm_cg_ip;
DELETE FROM dm_ggsn_board;
DELETE FROM dm_hplmn;
DELETE FROM dm_pcrf;
DELETE FROM dm_physical_port;
DELETE FROM dm_sgsn_ip;

  END clear;


  -- do a truncate on all the DM tables managed in this pack
  PROCEDURE clobber
  IS
  BEGIN
    NULL;
  END clobber;

  -- Fill all the DM tables managed in this pack
  PROCEDURE refill
  IS
  BEGIN
     	INSERT INTO dm_ggsn_board (dm_id, GGSN_DM_ID, start_time, end_time, GGSN_BOARD_KEY, GGSN_BOARD_ID)
         SELECT bipack_dm_seq.nextval, parent_dm_id, start_time, end_time, nc_id, GGSN_BOARD_ID
           FROM (SELECT dm.dm_id parent_dm_id,
                        nvl(greatest(nc.start_time, dm.start_time), nc.start_time) start_time,
                        nvl(least(nc.end_time, dm.end_time), nc.end_time) end_time,
                        nc.nc_id,
                        nc.GGSN_BOARD_ID
                   FROM DM_GGSN dm,
                        (SELECT nc_id, GGSN_BOARD_ID, timestamp start_time,
                                trunc(endstamp+30/24/60/60, 'MI')   end_time,
                                GGSN_ID
                           FROM NC_GGSN_BOARD) nc
                  WHERE nc.GGSN_ID = dm.GGSN_ID(+)
                    AND nc.start_time < dm.end_time(+)
                    AND nc.end_time  >= dm.start_time(+))
	WHERE start_time < end_time 
    ;

	INSERT INTO dm_cg_ip (dm_id, NETWORK_DM_ID, start_time, end_time, CG_IP_KEY, CG_IP_ID)
         SELECT bipack_dm_seq.nextval, parent_dm_id, start_time, end_time, nc_id, CG_IP_ID
           FROM (SELECT dm.dm_id parent_dm_id,
                        nvl(greatest(nc.start_time, dm.start_time), nc.start_time) start_time,
                        nvl(least(nc.end_time, dm.end_time), nc.end_time) end_time,
                        nc.nc_id,
                        nc.CG_IP_ID
                   FROM DM_NETWORK dm,
                        (SELECT nc_id, CG_IP_ID, timestamp start_time,
                                trunc(endstamp+30/24/60/60, 'MI')   end_time,
                                NETWORK_ID
                           FROM NC_CG_IP) nc
                  WHERE nc.NETWORK_ID = dm.NETWORK_ID(+)
                    AND nc.start_time < dm.end_time(+)
                    AND nc.end_time  >= dm.start_time(+))
	WHERE start_time < end_time 
    ;

	INSERT INTO dm_hplmn (dm_id, NETWORK_DM_ID, start_time, end_time, HPLMN_KEY, HPLMN_ID)
         SELECT bipack_dm_seq.nextval, parent_dm_id, start_time, end_time, nc_id, HPLMN_ID
           FROM (SELECT dm.dm_id parent_dm_id,
                        nvl(greatest(nc.start_time, dm.start_time), nc.start_time) start_time,
                        nvl(least(nc.end_time, dm.end_time), nc.end_time) end_time,
                        nc.nc_id,
                        nc.HPLMN_ID
                   FROM DM_NETWORK dm,
                        (SELECT nc_id, HPLMN_ID, timestamp start_time,
                                trunc(endstamp+30/24/60/60, 'MI')   end_time,
                                NETWORK_ID
                           FROM NC_HPLMN) nc
                  WHERE nc.NETWORK_ID = dm.NETWORK_ID(+)
                    AND nc.start_time < dm.end_time(+)
                    AND nc.end_time  >= dm.start_time(+))
	WHERE start_time < end_time 
    ;

	INSERT INTO dm_sgsn_ip (dm_id, SGSN_DM_ID, start_time, end_time, SGSN_IP_KEY, SGSN_IP_ID)
         SELECT bipack_dm_seq.nextval, parent_dm_id, start_time, end_time, nc_id, SGSN_IP_ID
           FROM (SELECT dm.dm_id parent_dm_id,
                        nvl(greatest(nc.start_time, dm.start_time), nc.start_time) start_time,
                        nvl(least(nc.end_time, dm.end_time), nc.end_time) end_time,
                        nc.nc_id,
                        nc.SGSN_IP_ID
                   FROM DM_SGSN dm,
                        (SELECT nc_id, SGSN_IP_ID, timestamp start_time,
                                trunc(endstamp+30/24/60/60, 'MI')   end_time,
                                SGSN_ID
                           FROM NC_SGSN_IP) nc
                  WHERE nc.SGSN_ID = dm.SGSN_ID(+)
                    AND nc.start_time < dm.end_time(+)
                    AND nc.end_time  >= dm.start_time(+))
	WHERE start_time < end_time 
    ;

	INSERT INTO dm_pcrf (dm_id, REGION_DM_ID, start_time, end_time, PCRF_KEY, PCRF_ID)
         SELECT bipack_dm_seq.nextval, parent_dm_id, start_time, end_time, nc_id, PCRF_ID
           FROM (SELECT dm.dm_id parent_dm_id,
                        nvl(greatest(nc.start_time, dm.start_time), nc.start_time) start_time,
                        nvl(least(nc.end_time, dm.end_time), nc.end_time) end_time,
                        nc.nc_id,
                        nc.PCRF_ID
                   FROM DM_REGION dm,
                        (SELECT nc_id, PCRF_ID, timestamp start_time,
                                trunc(endstamp+30/24/60/60, 'MI')   end_time,
                                REGION_ID
                           FROM NC_PCRF) nc
                  WHERE nc.REGION_ID = dm.REGION_ID(+)
                    AND nc.start_time < dm.end_time(+)
                    AND nc.end_time  >= dm.start_time(+))
	WHERE start_time < end_time 
    ;

	INSERT INTO dm_physical_port (dm_id, GGSN_DM_ID, start_time, end_time, PHYSICAL_PORT_KEY, PHYSICAL_PORT_ID)
         SELECT bipack_dm_seq.nextval, parent_dm_id, start_time, end_time, nc_id, PHYSICAL_PORT_ID
           FROM (SELECT dm.dm_id parent_dm_id,
                        nvl(greatest(nc.start_time, dm.start_time), nc.start_time) start_time,
                        nvl(least(nc.end_time, dm.end_time), nc.end_time) end_time,
                        nc.nc_id,
                        nc.PHYSICAL_PORT_ID
                   FROM DM_GGSN dm,
                        (SELECT nc_id, PHYSICAL_PORT_ID, timestamp start_time,
                                trunc(endstamp+30/24/60/60, 'MI')   end_time,
                                GGSN_ID
                           FROM NC_PHYSICAL_PORT) nc
                  WHERE nc.GGSN_ID = dm.GGSN_ID(+)
                    AND nc.start_time < dm.end_time(+)
                    AND nc.end_time  >= dm.start_time(+))
	WHERE start_time < end_time 
    ;

	
  END refill;

  PROCEDURE collect_stats
  IS
	schemaUser VARCHAR2(50);
  BEGIN
	SELECT sys_context('USERENV', 'CURRENT_SCHEMA') INTO schemaUser from dual;
    FOR tables IN (SELECT table_name FROM user_tables)
    LOOP
      dbms_stats.gather_table_stats(
        ownname    => schemaUser,
        tabname    => tables.table_name,
        method_opt => 'for all columns size auto',
        cascade    => TRUE);
    END LOOP;
  END collect_stats;

  PROCEDURE refill_hierarchies
  IS
  BEGIN
    NULL;
  END refill_hierarchies;


END bi_data;
/


